#define CLK (1<<0) // bqps
#define LATCH (1<<1) // y 88..
#define DATA (1<<2) // r

#define MAGIC_BOOT_KEY            0xDC42ACCA
#define BOOTLOADER_START_ADDRESS  (FLASH_SIZE_BYTES - BOOTLOADER_SEC_SIZE_BYTES)

#define R1 (1 << 0)
#define L1 (1 << 1)
#define X1 (1 << 2)
#define A1 (1 << 3)
#define EAST1 (1 << 4)
#define WEST1 (1 << 5)
#define SOUTH1 (1 << 6)
#define NORTH1 (1 << 7)
#define START1 (1 << 8)
#define SELECT1 (1 << 9)
#define Y1 (1 << 10)
#define B1 (1 << 11)

#define held 0
#define new 1
#define rel 2

bool _A[3];
bool _B[3];
bool _X[3];
bool _Y[3];
bool _Up[3];
bool _Down[3];
bool _Left[3];
bool _Right[3];
bool _Start[3];
bool _Select[3];
bool _L[3];
bool _R[3];

unsigned int CompletePad, ExPad, TempPad;

void UPDATEPAD(int pad, int var){
	_R[pad] = (var & R1);
	_L[pad] = (var & L1) >> 1;
	_X[pad] = (var & X1) >> 2;
	_A[pad] = (var & A1) >> 3;
	_Right[pad] = (var & EAST1) >> 4;
	_Left[pad] = (var & WEST1) >> 5;
	_Down[pad] = (var & SOUTH1) >> 6;
	_Up[pad] = (var & NORTH1) >> 7;
	_Start[pad] = (var & START1) >> 8;
	_Select[pad] = (var & SELECT1) >> 9;
	_Y[pad] = (var & Y1) >> 10;
	_B[pad] = (var & B1) >> 11;
}

void UpdatePad(int joy_code) 
{
	ExPad = CompletePad;
	CompletePad = joy_code;
	
	UPDATEPAD(held, CompletePad); // held
	UPDATEPAD(rel, (ExPad & (~CompletePad))); // released
	UPDATEPAD(new, (CompletePad & (~ExPad))); // newpress
}

uint16_t joypad_code = 0xFFFF;

void retropadLoop (void)
{
		joypad_code = 0;
		PORTD |= LATCH;
		_delay_us(12);
		PORTD &= ~LATCH;

		int numBits = 16;
		for (int i = 0; i < numBits; i++) {
			_delay_us(6);
			PORTD &= ~CLK;
			joypad_code <<= 1;
			if (!(PIND & DATA)) { joypad_code |= 1; }
			_delay_us(6);
			PORTD |= CLK;
		}
		joypad_code >>= 4;
}

void SNESPAD_Init(void)
{
	DDRD &= ~(DATA);
	PORTD |= (DATA);
	DDRD |= (CLK | LATCH);
	PORTD &= ~(CLK | LATCH);
}
